@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2010 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -32,21 +32,29 @@
 #include "tun.h"
 #include "misc.h"
 
-#define MAX_ROUTES 100
+#define MAX_ROUTES_DEFAULT 100
 
 #ifdef WIN32
 /*
  * Windows route methods
  */
-#define ROUTE_METHOD_IPAPI  0  /* use IP helper API */
-#define ROUTE_METHOD_EXE    1  /* use route.exe */
-#define ROUTE_METHOD_MASK   1
+#define ROUTE_METHOD_ADAPTIVE  0  /* try IP helper first then route.exe */
+#define ROUTE_METHOD_IPAPI     1  /* use IP helper API */
+#define ROUTE_METHOD_EXE       2  /* use route.exe */
+#define ROUTE_METHOD_MASK      3
 #endif
 
 /*
  * Route add flags (must stay clear of ROUTE_METHOD bits)
  */
-#define ROUTE_DELETE_FIRST  2
+#define ROUTE_DELETE_FIRST  4
+
+struct route_bypass
+{
+# define N_ROUTE_BYPASS 8
+  int n_bypass;
+  in_addr_t bypass[N_ROUTE_BYPASS];
+};
 
 struct route_special_addr
 {
@@ -56,6 +64,9 @@ struct route_special_addr
   bool net_gateway_defined;
   in_addr_t remote_host;
   bool remote_host_defined;
+  struct route_bypass bypass;
+  int default_metric;
+  bool default_metric_defined;
 };
 
 struct route_option {
@@ -65,12 +76,20 @@ struct route_option {
   const char *metric;
 };
 
+/* redirect-gateway flags */
+#define RG_ENABLE         (1<<0)
+#define RG_LOCAL          (1<<1)
+#define RG_DEF1           (1<<2)
+#define RG_BYPASS_DHCP    (1<<3)
+#define RG_BYPASS_DNS     (1<<4)
+#define RG_REROUTE_GW     (1<<5)
+#define RG_AUTO_LOCAL     (1<<6)
+
 struct route_option_list {
+  unsigned int flags;
+  int capacity;
   int n;
-  bool redirect_default_gateway;
-  bool redirect_local;
-  bool redirect_def1;
-  struct route_option routes[MAX_ROUTES];
+  struct route_option routes[EMPTY_ARRAY_SIZE];
 };
 
 struct route {
@@ -86,13 +105,12 @@ struct route {
 struct route_list {
   bool routes_added;
   struct route_special_addr spec;
-  bool redirect_default_gateway;
-  bool redirect_local;
-  bool redirect_def1;
+  unsigned int flags;
   bool did_redirect_default_gateway;
-
+  bool did_local;
+  int capacity;
   int n;
-  struct route routes[MAX_ROUTES];
+  struct route routes[EMPTY_ARRAY_SIZE];
 };
 
 #if P2MP
@@ -104,9 +122,13 @@ struct iroute {
 };
 #endif
 
-struct route_option_list *new_route_option_list (struct gc_arena *a);
+struct route_option_list *new_route_option_list (const int max_routes, struct gc_arena *a);
+struct route_option_list *clone_route_option_list (const struct route_option_list *src, struct gc_arena *a);
+void copy_route_option_list (struct route_option_list *dest, const struct route_option_list *src);
 
-struct route_list *new_route_list (struct gc_arena *a);
+struct route_list *new_route_list (const int max_routes, struct gc_arena *a);
+
+void add_route (struct route *r, const struct tuntap *tt, unsigned int flags, const struct env_set *es);
 
 void add_route_to_option_list (struct route_option_list *l,
 			       const char *network,
@@ -114,14 +136,17 @@ void add_route_to_option_list (struct route_option_list *l,
 			       const char *gateway,
 			       const char *metric);
 
-void clear_route_list (struct route_list *rl);
-
 bool init_route_list (struct route_list *rl,
 		      const struct route_option_list *opt,
 		      const char *remote_endpoint,
+		      int default_metric,
 		      in_addr_t remote_host,
 		      struct env_set *es);
 
+void route_list_add_default_gateway (struct route_list *rl,
+				     struct env_set *es,
+				     const in_addr_t addr);
+
 void add_routes (struct route_list *rl,
 		 const struct tuntap *tt,
 		 unsigned int flags,
@@ -134,6 +159,25 @@ void delete_routes (struct route_list *rl,
 
 void setenv_routes (struct env_set *es, const struct route_list *rl);
 
+bool is_special_addr (const char *addr_str);
+
+bool get_default_gateway (in_addr_t *ip, in_addr_t *netmask);
+
+/*
+ * Test if addr is reachable via a local interface (return ILA_LOCAL),
+ * or if it needs to be routed via the default gateway (return
+ * ILA_NONLOCAL).  If the current platform doesn't implement this
+ * function, return ILA_NOT_IMPLEMENTED.
+ */
+#define TLA_NOT_IMPLEMENTED 0
+#define TLA_NONLOCAL        1
+#define TLA_LOCAL           2
+int test_local_addr (const in_addr_t addr);
+
+#if AUTO_USERID || defined(ENABLE_PUSH_PEER_INFO)
+bool get_default_gateway_mac_addr (unsigned char *macaddr);
+#endif
+
 #ifdef ENABLE_DEBUG
 void print_route_options (const struct route_option_list *rol,
 			  int level);
@@ -164,4 +208,13 @@ netbits_to_netmask (const int netbits)
   return mask;
 }
 
+static inline bool
+route_list_default_gateway_needed (const struct route_list *rl)
+{
+  if (!rl)
+    return false;
+  else
+    return !rl->spec.remote_endpoint_defined;
+}
+
 #endif
